;***************************************************************
; project name:    PCI Device Enumeration
; target platform: MenuetOS, x86 (IA-32), x86-64 achitectures
; compiler:        flat assmebler 1.60
; version:         1.11
; last update:     3th June 2005
; maintained by:   Sergey Kuzmin aka Wildwest
; e-mail:          kuzmin_serg@list.ru
; project site:    http://coolthemes.narod.ru/pcidev.html
;***************************************************************
;Summary: This program will attempt to scan the PCI Bus
;        and display basic information about each device
;        connected to the PCI Bus.
;***************************************************************
;HISTORY:
;1.11: PCIDEV   06/03/2005
;Author:     Sergey Kuzmin aka Wildwest <kuzmin_serg@list.ru>
;Features:  added quantity of devices,
;           detection of Company Name based on Vendor ID,
;           database of VenID (35 ID's),
;           macros.inc for smaller size,
;           interface+(skinned window),
;           VenID before DevID in 'table'(was DevID before VenID)
;----------------------------------------------------------------
;1.0: PCIDEV    01/30/2005
;Author:    Jason Delozier
;Features:  able to detect PCI version, quantity of PCI buses,
;           Vendor&Device ID for appropriate Device on Bus;
;           detect Revision, Class and Subclass of Device,
;           and make Description based on Class
;-------------------------------------------------------------

use32

	       org    0x0

	       db     'MENUET01'	      ; 8 byte id
	       dd     0x01		      ; header version
	       dd     START		      ; start of code
	       dd     I_END		      ; size of image
	       dd     0x1000		      ; memory for app
	       dd     0x1000		      ; esp
	       dd     0x0 , 0x0 	      ; I_Param , I_Icon

include 'macros.inc'

START:				; start of execution
     call draw_window

still:
    mov  eax,10 		; wait here for event
    int  0x40

    cmp  eax,1			; redraw request ?
    je	 red
    cmp  eax,2			; key in buffer ?
    je	 key
    cmp  eax,3			; button in buffer ?
    je	 button

    jmp  still

  red:				; redraw
    call draw_window
    jmp  still
  key:				; key
    mov  eax,2			; just read it and ignore
    int  0x40
    jmp  still
  button:			; button
    mov  eax,17 		; get id
    int  0x40

    cmp  ah,1			; button id=1 ?
    jne  noclose

    mov  eax,-1 		; close this program
    int  0x40
  noclose:
    jmp  still




;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************


draw_window:
    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,1			   ; 1, start of draw
    int  0x40

				   ; DRAW WINDOW
    mov  eax,0			   ; function 0 : define and draw window
    mov  ebx,100*65536+550	   ; [x start] *65536 + [x size]
    mov  ecx,100*65536+550	   ; [y start] *65536 + [y size]
    mov  edx,0x03ffffff 	   ; color of work area RRGGBB,8->color gl
    mov  esi,0x805080d0 	   ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x005080d0 	   ; color of frames    RRGGBB
    int  0x40

				   ; WINDOW LABEL
    mov  eax,4			   ; function 4 : write text to window
    mov  ebx,8*65536+8		   ; [x start] *65536 + [y start]
    mov  ecx,0x10ddeeff 	   ; font 1 & color ( 0xF0RRGGBB )
    mov  edx,labelt		   ; pointer to text beginning
    mov  esi,labellen-labelt	   ; text length
    int  0x40

    mov [total],0

    mov ebx, 20*65536+25	   ;Draw work format
    mov ecx, 0x224466
    mov edx, PCIWin
    mov esi, 78 ; lenght of line (74)
newline:
    mov eax, 4
    int 0x40
    add ebx, 10
    add edx, 78  ; lenght of line (74)
    cmp [edx], byte 'x'
    jne newline


    call Get_PCI_Info
    mov cx, [PCI_Version]
    mov eax, 47
    mov ebx, 0x00040100
    mov esi, 0x00000000
    mov edx, 110*65536+45
    int 0x40

    mov cl, [PCI_LastBus]
    mov eax, 47
    mov ebx, 0x00020100
    mov esi, 0x00000000
    mov edx, 110*65536+55
    int 0x40

   call scan_Pci

    xor ecx, ecx
	mov cl, [total]
    mov eax, 47
    mov ebx, 0x00020000
    mov esi, 0x00000000
    mov edx, 150*65536+65
    int 0x40


    mov  eax,12 		   ; function 12:tell os about windowdraw
    mov  ebx,2			   ; 2, end of draw
    int  0x40

    ret

Get_PCI_Info:
    mov eax, 62
    mov ebx, 0
    int 0x40
    mov [PCI_Version], ax

    mov eax, 62
    mov ebx, 1
    int 0x40
    mov [PCI_LastBus], al
ret



scan_Pci:
   cmp [PCI_LastBus],0xff
   jne Pci_Exists
   ret
Pci_Exists:
   mov [V_Bus], 0	  ;reset varibles
   mov [V_Dev], 0	  ;
   mov edx,  20*65536+110 ;start write position

Start_Enum:
   mov bl, 6	     ;get Device ID / Vendor ID
   mov bh, [V_Bus]
   mov ch, [V_Dev]
   mov cl, 0
   mov eax, 62
   int 0x40

   cmp ax, 0	     ;check next slot if nothing exists
   je nextDev
   cmp ax, 0xffff
   je nextDev

   mov [PCI_Vendor], ax
   shr eax, 16
   mov [PCI_Device], ax

   mov eax, 62		  ;PCI Sys Function
   mov bl, 4		  ;Read config byte
   mov bh, [V_Bus]	  ;Bus #
   mov ch, [V_Dev]	  ;Device # on bus
   mov cl, 0x08 	  ;Register to read (Get Revision)
   int 0x40		  ;Read it
   mov [PCI_Rev], al	  ;Save it

   mov eax, 62		  ;PCI Sys Function
   mov cl, 0x0b 	  ;Register to read (Get class)
   int 0x40		  ;Read it
   mov [PCI_Class], al	  ;Save it

   mov eax, 62		  ;PCI Sys Function
   mov cl, 0x0a 	  ;Register to read (Get Subclass)
   int 0x40		  ;Read it
   mov [PCI_SubClass], al ;Save it

   call Print_New_Device

nextDev:
   inc [V_Dev]
   jno Start_Enum    ;jump if no overflow (<256)

   mov [V_Dev], 0
   inc [V_Bus]
   mov al, [PCI_LastBus]

   cmp byte [V_Bus], al
   jbe Start_Enum
ret

Print_New_Device:
   inc [total]

 ;  mov eax, 47    ;Write number to screen
 ;  mov ebx, 0x00040100    ;4 byte number, print in hexidecimal
 ;  mov esi, 0x00000000    ;Color of text
 ;  mov cx, total   ;Pointer to number to be written
 ;  int 0x40               ;Write it

   mov eax, 47	  ;Write number to screen
   mov ebx, 0x00040100	  ;4 byte number, print in hexidecimal
   mov esi, 0x00000000	  ;Color of text
   mov cx, [PCI_Vendor]   ;Pointer to number to be written
   int 0x40		  ;Write it

   push dx		  ;Calculate position
   mov dx, 62		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cx, [PCI_Device]   ;Write Vendor ID
   int 0x40		  ;

   mov ebx, 0x00020100	  ;2 byte number, in hexidecimal format
   push dx		  ;move write cursor
   mov dx, 110		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cl, [V_Bus]	  ;write bus number
   int 0x40		  ;

   push dx		  ;move write cursor
   mov dx, 146		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cl, [V_Dev]	  ;Write device number
   int 0x40		  ;

   push dx		  ;move write cursor
   mov dx, 179		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cl, [PCI_Rev]	  ;Draw Revision to screen
   int 0x40		  ;

   push dx		  ;move write cursor
   mov dx, 215		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cl, [PCI_Class]	  ;Draw Class to screen
   int 0x40		  ;

   push dx		  ;move write curson
   mov dx, 266		  ;
   shl edx, 16		  ;
   pop dx		  ;
   mov cl, [PCI_SubClass] ;Draw Sub Class to screen
   int 0x40		  ;


			  ;Write Names
   push dx		  ;remember our current write line
   mov esi, 17		  ;length of text
   mov bx, 310		  ;X-position of text on screen
   shl ebx, 16		  ;
   mov bx, dx		  ;Current Y position
   mov ecx, 0		  ;color of text

   mov [cur], dx

   mov [r],0
cmp [PCI_Vendor], $10DE; nVidia
   jz ex
   inc [r]
cmp [PCI_Vendor], $11C1; Lucent
   jz ex
   inc [r]
cmp [PCI_Vendor], $147B; Abit
   jz ex
   inc [r]
cmp [PCI_Vendor], $1131; Philips
   jz ex
   inc [r]
cmp [PCI_Vendor], $1095; Silicon Image
   jz ex
   inc [r]
cmp [PCI_Vendor], $1002; ATI
   jz ex
   inc [r]
cmp [PCI_Vendor], $1462  ;MSI
   jz ex
   inc [r]
cmp [PCI_Vendor], $1106  ;VIA
   jz ex
   inc [r]
cmp [PCI_Vendor], $8086  ;Intel
   jz ex
   inc [r]
cmp [PCI_Vendor], $1039  ;SIS
   jz ex
   inc [r]
cmp [PCI_Vendor], $10EC  ;Realtek
   jz ex
   inc [r]
cmp [PCI_Vendor], $1022  ;AMD
   jz ex
   inc [r]
cmp [PCI_Vendor], $5333  ;S3
   jz ex
   inc [r]
cmp [PCI_Vendor], $10B7  ;3Com N
   jz ex
   inc [r]
cmp [PCI_Vendor], $12B9  ;3Com M
   jz ex
   inc [r]
cmp [PCI_Vendor], $10B9  ;ALi
   jz ex
   inc [r]
cmp [PCI_Vendor], $1695  ;Epox
   jz ex
   inc [r]
cmp [PCI_Vendor], $1043  ;Asus
   jz ex
   inc [r]
cmp [PCI_Vendor], $1014  ;IBM
   jz ex
   inc [r]
cmp [PCI_Vendor], $1461  ;Avermedia
   jz ex
   inc [r]
cmp [PCI_Vendor], $121A  ;3Dfx
   jz ex
   inc [r]
cmp [PCI_Vendor], $102b  ;Matrox
   jz ex
   inc [r]
cmp [PCI_Vendor], $12D2  ; Nvidia (Was:STB,SGS Thompson)
   jz ex
   inc [r]
cmp [PCI_Vendor], $13F6  ;C-Media
   jz ex
   inc [r]
cmp [PCI_Vendor], $1274  ;Creative Labs, Ensoniq
   jz ex
   inc [r]
cmp [PCI_Vendor], $1102  ;Creative Labs
   jz ex
   inc [r]
cmp [PCI_Vendor], $1019  ;ECS
   jz ex
   inc [r]
cmp [PCI_Vendor], $1458  ;Gigabyte ?
   jz ex
   inc [r]
cmp [PCI_Vendor], $17F2  ;Albatron
   jz ex
   inc [r]
cmp [PCI_Vendor], $1076  ;Chaintech
   jz ex
   inc [r]
cmp [PCI_Vendor], $1849  ;ASRock
   jz ex
   inc [r]
cmp [PCI_Vendor], $4444  ;Conexant
   jz ex
   inc [r]
cmp [PCI_Vendor], $166d  ;Broadcom
   jz ex
   inc [r]
cmp [PCI_Vendor], $127A  ;Conexant, Rockwell
   jz ex
   inc [r]
cmp [PCI_Vendor], $9004 ;Adaptec
   jz ex
   inc [r]

  ex:

   xor eax, eax 	  ;calculate pointer to string
   mov al, [r]		  ;Determine Description based on class
   mov dl,  17		  ;Each Name is 22 characters long
   mul dl		  ;get offset to correct description
   mov edx, Vendorlist	  ;starting point of description text
   add edx, eax 	  ;calculated corresponding description
   mov eax, 4		  ;draw text sys function
   int 0x40		  ;draw it
;------------------------------------------------------------------

   mov esi, 26		  ;length of text
   mov bx, 420		  ;X-position of text on screen
   shl ebx, 16		  ;
   mov bx, [cur]	  ;Current Y position
   mov ecx, 0		  ;color of text


   xor eax, eax 	  ;calculate pointer to string
   mov al, [PCI_Class]	  ;Determine Description based on class
   mov dl,  26		  ;Each description is 26 characters long
   mul dl		  ;get offset to correct description
   mov edx, Descriptions  ;starting point of description text
   add edx, eax 	  ;calculated corresponding description
   mov eax, 4		  ;draw text sys function
   int 0x40

   mov dx, 20		  ;Starting Y coordinate
   shl edx, 16		  ;
   pop dx		  ;
   add edx, 10		  ;Next line down


ret



; DATA AREA
V_Bus	     db 0
V_Dev	     db 0
PCI_Version  dw 0
PCI_LastBus  db 0
PCI_Device   dw 0
PCI_Vendor   dw 0
PCI_Bus      db 0
PCI_Dev      db 0
PCI_Rev      db 0
PCI_Class    db 0
PCI_SubClass db 0

labelt:
       db 'PCI Device Enumeration v 1.11 by J. Delozier and S. Kuzmin'
labellen:

r db ?
cur dw ?
total db ?

Vendorlist:
db 'Nvidia Corp.     '
db 'Lucent/Agere     '
db 'Abit Computer    '
db 'Philips Semicond.'
db 'Silicon Image    '
db 'ATI Technologies '
db 'Micro-Star Int.  '
db 'VIA Technologies '
db 'Intel Corporation'
db 'SIS              '
db 'Realtek Semicond.'
db 'AMD              '
db 'S3 Inc           '
db '3COM Corp,Network'
db '3COM Corp,Modem  '
db 'Acer Labs Incorp.'
db 'Epox Computer Co '
db 'ASUSTeK Computer '
db 'IBM              '
db 'Avermedia Tech.  '
db '3Dfx Interactive '
db 'Matrox Graphics  '
db 'Nvidia- Thompson '
db 'C-Media Electron.'
db 'Creative-Ensoniq '
db 'Creative Labs    '
db 'ECS              '
db 'Giga-Byte Tech.  '
db 'Albatron Corp    '
db 'Chaintech Comp.  '
db 'ASRock Inc       '
db 'Conexant-iCompres'
db 'Broadcom -SiByte)'
db 'Rockwell-Conexant'
db 'Adaptec Inc      '
db 'Unknown vendor   '


PCIWin:
db 'Please remember to enable PCI Access to Applications in Setup Menu.           '
db '                                                                              '
db 'PCI Version  =                                                                '
db 'Last PCI Bus =                                                                '
db 'Quantity of devices =                                                         '
db '                                                                              '
db 'VenID  DevID  Bus#  Dev#  Rev  Class  Subclass     Company       Description  '
db '-----  -----  ----  ----  ---  -----  --------  -------------  ---------------'
db 'x'

Descriptions:
db 'Unknown                   '
db 'Storage                   '
db 'Network                   '
db 'Display                   '
db 'Multimedia                '
db 'Memory                    '
db 'Bridge                    '
db 'Simple Communication      '
db 'System                    '
db 'Input                     '
db 'Docking                   '
db 'Processor                 '
db 'Serial                    '
db 'Wireless                  '
db 'Intelligent I/O           '
db 'Satellite Communication   '
db 'En/Decryption             '
db 'Data Aqu & Signal Process '

I_END: